@@ -6,14 +6,27 @@ import re
 import shutil
 import tempfile
 import urllib
-import zc.recipe.egg
 import zipfile
+from zc.buildout.buildout import Options
+from zc.recipe.egg import Eggs, Scripts
 
 logger = logging.getLogger(__name__)
 
 rx_setuptools = re.compile(
     '^.*' + re.escape(os.sep) + '(setuptools|distribute)-.*?-py\d.\d.egg$')
 
+dev_appserver_initialization = '''
+import os
+
+def mkvar():
+  var = %(var)r
+  if not os.path.exists(var):
+    os.mkdir(var)
+  return var
+os.environ['TMPDIR'] = mkvar()
+
+from dev_appserver import *
+'''
 
 def copytree(src, dst, symlinks=0, allowed_basenames=None, exclude=[]):
     """Local implementation of shutil's copytree function.
@@ -39,7 +52,7 @@ def copytree(src, dst, symlinks=0, allowed_basenames=None, exclude=[]):
         if allowed_basenames:
             if os.path.isfile(srcname):
                 if name not in allowed_basenames:
-                    logger.debug("Skipped %s" % srcname) 
+                    logger.debug("Skipped %s" % srcname)
                     continue
         if os.path.basename(srcname) in exclude:
             continue
@@ -61,7 +74,7 @@ def copytree(src, dst, symlinks=0, allowed_basenames=None, exclude=[]):
             elif not symlinks:
                 shutil.copy2(srcname, dstname)
         except (IOError, os.error), why:
-            logging.error("Can't copy %s to %s: %s" % 
+            logging.error("Can't copy %s to %s: %s" %
                           (srcname, dstname, str(why)))
 
 
@@ -94,7 +107,7 @@ class Zipper(object):
             os.path.realpath(fname), archivename, compression_type)
 
 
-class Recipe(zc.recipe.egg.Eggs):
+class Recipe(Scripts):
     """Buildout recipe for Google App Engine."""
 
     def __init__(self, buildout, name, opts):
@@ -104,67 +117,50 @@ class Recipe(zc.recipe.egg.Eggs):
         opts['app-directory'] = os.path.join(buildout['buildout']
                                              ['parts-directory'],
                                              self.name)
-        opts['bin-directory'] = buildout['buildout']['bin-directory']
+        # avoid warnings
+        opts.get('entry-points')
+        opts.get('arguments')
 
-    def write_server_script(self, name, bin, sys_path):
+    def write_server_script(self, name, bin):
         """Generates bin script with given name."""
 
-        # Open the destination script file
-        path = os.path.join(self.options['bin-directory'], name)
-        script = open(path, 'w')
+        var = os.path.join(self.buildout['buildout']['parts-directory'], '%s_var' % self.name)
+        if not os.path.isdir(var):
+            os.makedirs(var)
 
-        # Write script
-        script.write("#!%s\n\n" % 
-                        self.buildout[self.buildout['buildout']['python']]
-                        ['executable'])
-        script.write("import sys\nimport os\n\n")
-        script.write("sys.path[0:0] = [\n")
-        script.write(",\n".join(["  %r" % p for p in sys_path]))
-        script.write("\n  ]\n\n")
-        script.write("PROJECT_HOME = %r\n\n" %
-                     self.buildout['buildout']['directory'])
-        script.write("def mkvar():\n")
-        script.write("  var = os.path.join(PROJECT_HOME, 'var')\n")
-        script.write("  if not os.path.exists(var):\n")
-        script.write("    os.mkdir(var)\n")
-        script.write("  return var\n\n")
-        script.write("from dev_appserver import *\n\n")
-        script.write("if __name__ == '__main__':\n")
-        script.write("  os.environ['TMPDIR'] = mkvar()\n")
-        script.write("  run_file(%r, locals())" % bin)
+        options = self.options.copy()
+        options['eggs'] = ''
+        options['entry-points'] = '%s=dev_appserver:run_file' % name
+        options['initialization'] = dev_appserver_initialization % dict(var=var)
+        options['initialization'] += '\n' + self.options.get('initialization', '')
+        options['arguments'] = '%r, locals()' % bin
+        options = Options(self.buildout, self.name, options)
+        scripts = Scripts(self.buildout, self.name, options)
+        scripts.install()
 
-        # Close script file and modify permissions
-        script.close()
-        os.chmod(path, 0755)
-
-    def write_appcfg_script(self, bin, sys_path):
+    def write_appcfg_script(self, bin):
         """Generates the app configuration script in bin."""
+        options = self.options.copy()
+        options['eggs'] = ''
+        options['entry-points'] = 'appcfg=appcfg:run_file'
+        options['arguments'] = '%r, locals()' % bin
+        options = Options(self.buildout, self.name, options)
+        scripts = Scripts(self.buildout, self.name, options)
+        scripts.install()
 
-        # Open the destination script file
-        path = os.path.join(self.options['bin-directory'], 'appcfg')
-        script = open(path, 'w')
-
-        # Write script
-        script.write("#!%s\n\n" % 
-                        self.buildout[self.buildout['buildout']['python']]
-                        ['executable'])
-        script.write("import sys\nimport os\n\n")
-        script.write("sys.path[0:0] = [\n")
-        script.write(",\n".join(["  '%s'" % p for p in sys_path]))
-        script.write("\n  ]\n\n")
-        script.write("from appcfg import *\n\n")
-        script.write("if __name__ == '__main__':\n")
-        script.write("  run_file('%s', locals())" % bin)
-
-        # Close script file and modify permissions
-        script.close()
-        os.chmod(path, 0755)
+    def write_extra_scripts(self):
+        options = self.options.copy()
+        options['eggs'] = ''
+        options = Options(self.buildout, self.name, options)
+        scripts = Scripts(self.buildout, self.name, options)
+        scripts.install()
 
     def install_appengine(self):
         """Downloads and installs Google App Engine."""
         arch_filename = self.options['url'].split('/')[-1].split(os.sep)[-1]
         dst = os.path.join(self.buildout['buildout']['parts-directory'])
         downloads_dir = os.path.join(os.getcwd(), 'downloads')
+        downloads_dir = self.buildout['buildout'].get('download-cache', downloads_dir)
         if not os.path.isdir(downloads_dir):
             os.mkdir(downloads_dir)
         src = os.path.join(downloads_dir, arch_filename)
@@ -184,23 +180,27 @@ class Recipe(zc.recipe.egg.Eggs):
                 outfile.write(arch.read(name))
                 outfile.close()
 
-    def setup_bin(self, ws):
+    def setup_bin(self):
         """Setup bin scripts."""
         gae = self.options.get('appengine-lib')
         if gae is None:
-            gae = os.path.join(self.buildout['buildout']['parts-directory'], 
+            gae = os.path.join(self.buildout['buildout']['parts-directory'],
                                'google_appengine')
-        sys_path = [gae]
+
+        extra_paths = self.options.get('extra-paths', '')
+        extra_paths += '\n' + gae
+        self.options['extra-paths'] = extra_paths
 
         # Write server script
         gae_server = os.path.join(gae, 'dev_appserver.py')
-        self.write_server_script(self.options.get('server-script', self.name),  
-                                 gae_server,
-                                 sys_path)
+        self.write_server_script(self.options.get('server-script', self.name),
+                                 gae_server)
 
         # Write app configuration script
         gae_config = os.path.join(gae, 'appcfg.py')
-        self.write_appcfg_script(gae_config, sys_path)
+        self.write_appcfg_script(gae_config)
+
+        self.write_extra_scripts()
 
     def write_pkg_resources_stub(self, d):
         """Writes a stub for setuptool's pkg_resources module."""
@@ -311,10 +311,10 @@ class Recipe(zc.recipe.egg.Eggs):
     def install(self):
         """Creates the part."""
         options = self.options
-        reqs, ws = self.working_set()
         if not self.options.get('appengine-lib', False):
             self.install_appengine()
-        self.setup_bin(ws)
+        self.setup_bin()
+        reqs, ws = self.working_set()
         app_dir = options['app-directory']
         if options.get('zip-packages', 'YES').lower() in ['yes', 'true']:
             temp_dir = os.path.join(tempfile.mkdtemp(), self.name)
@@ -324,7 +324,7 @@ class Recipe(zc.recipe.egg.Eggs):
             shutil.rmtree(app_dir, True)
         if not os.path.exists(app_dir):
             os.mkdir(app_dir)
-        self.copy_packages(ws, temp_dir) 
+        self.copy_packages(ws, temp_dir)
         if options.get('zip-packages', 'YES').lower() in ['yes', 'true']:
             self.zip_packages(ws, temp_dir)
             if os.path.isdir(temp_dir):
@@ -341,8 +341,8 @@ class Recipe(zc.recipe.egg.Eggs):
     def update(self):
         """Updates the part."""
         options = self.options
+        self.setup_bin()
         reqs, ws = self.working_set()
-        self.setup_bin(ws)
         self.copy_sources()
         patch_file = options.get('patch')
         patch_options = options.get('patch_options')
