@@ -8,7 +8,7 @@ setup (
     name='bazpkg',
     version='0.1.0',
     author="Someone Else",
-    description="Fer testing purposes only.",
+    description="For testing purposes only.",
     long_description="",
     license="Apache License 2.0",
     keywords="dummy",
